@@ -19,7 +19,7 @@ class TestReadWrite(unittest.TestCase):
 
 
         nvec = 2
-        ashape = (2,3)
+        ashape=(21,21)
         Sdtype = 'S6'
         # all currently available types, scalar, 1-d and 2-d array columns
         dtype=[('u1scalar','u1'),
@@ -251,6 +251,32 @@ class TestReadWrite(unittest.TestCase):
             if os.path.exists(fname):
                 os.remove(fname)
  
+
+    def testImageSlice(self):
+        fname=tempfile.mktemp(prefix='fitsio-ImageSlice-',suffix='.fits')
+        try:
+            with fitsio.FITS(fname,'rw',clobber=True) as fits:
+                # note mixing up byte orders a bit
+                for dtype in ['u1','i1','u2','i2','<u4','i4','i8','>f4','f8']:
+                    data = numpy.arange(16*20,dtype=dtype).reshape(16,20)
+                    header={'DTYPE':dtype,'NBYTES':data.dtype.itemsize}
+                    fits.write_image(data, header=header)
+                    rdata = fits[-1][4:12, 9:17]
+
+                    self.compare_array(data[4:12,9:17], rdata, "images")
+
+                    rh = fits[-1].read_header()
+                    for k,v in header.iteritems():
+                        rv = rh[k]
+                        if isinstance(rv,str):
+                            v = v.strip()
+                            rv = rv.strip()
+                        self.assertEqual(v,rv,"testing equal key '%s'" % k)
+        finally:
+            if os.path.exists(fname):
+                os.remove(fname)
+
+
     def testRiceTileCompressedWriteRead(self):
         """
         Test a basic image write, data and a header, then reading back in to
@@ -590,28 +616,235 @@ class TestReadWrite(unittest.TestCase):
             # now test read_column
             with fitsio.FITS(fname) as fits:
 
+                for f in self.data.dtype.names:
+                    d = fits[1].read_column(f)
+                    self.compare_array(self.data[f], d, "table 1 single field read '%s'" % f)
+
                 for f in self.data2.dtype.names:
                     d = fits['newext'].read_column(f)
-                    self.compare_array(self.data2[f], d, "table single field read '%s'" % f)
+                    self.compare_array(self.data2[f], d, "table 2 single field read '%s'" % f)
 
                 # now list of columns
-                cols=['u2scalar','f4vec','Sarr']
-                d = fits[1].read(columns=cols)
-                for f in d.dtype.names: 
-                    self.compare_array(self.data[f][:], d[f], "test column list %s" % f)
+                for cols in [['u2scalar','f4vec','Sarr'],
+                             ['f8scalar','u2arr','Sscalar']]:
+                    d = fits[1].read(columns=cols)
+                    for f in d.dtype.names: 
+                        self.compare_array(self.data[f][:], d[f], "test column list %s" % f)
 
 
-                cols=['u2scalar','f4vec','Sarr']
-                rows = [1,3]
-                d = fits[1].read(columns=cols, rows=rows)
-                for f in d.dtype.names: 
-                    self.compare_array(self.data[f][rows], d[f], "test column list %s row subset" % f)
+                    rows = [1,3]
+                    d = fits[1].read(columns=cols, rows=rows)
+                    for f in d.dtype.names: 
+                        self.compare_array(self.data[f][rows], d[f], "test column list %s row subset" % f)
 
         finally:
             if os.path.exists(fname):
+                #pass
                 os.remove(fname)
 
 
+    def testTableWriteDictOfArraysScratch(self):
+        """
+        This version creating the table from the names and list
+        Test a basic table write, data and a header, then reading back in to
+        check the values
+        """
+
+        fname=tempfile.mktemp(prefix='fitsio-TableDict-',suffix='.fits')
+        try:
+            with fitsio.FITS(fname,'rw',clobber=True) as fits:
+
+                try:
+                    d={}
+                    for n in self.data.dtype.names:
+                        d[n] = self.data[n]
+
+                    fits.write(d)
+                    write_success=True
+                except:
+                    write_success=False
+
+                self.assertTrue(write_success,"write should not raise an error")
+                if not write_success:
+                    skipTest("cannot test result if write failed")
+
+            d = fitsio.read(fname)
+            self.compare_rec(self.data, d, "list of dicts, scratch")
+
+        finally:
+            if os.path.exists(fname):
+                #pass
+                os.remove(fname)
+
+    def testTableWriteDictOfArrays(self):
+        """
+        This version creating the table from the names and list
+        Test a basic table write, data and a header, then reading back in to
+        check the values
+        """
+
+        fname=tempfile.mktemp(prefix='fitsio-TableDict-',suffix='.fits')
+        try:
+            with fitsio.FITS(fname,'rw',clobber=True) as fits:
+
+                try:
+                    fits.create_table_hdu(self.data, extname='mytable')
+
+                    d={}
+                    for n in self.data.dtype.names:
+                        d[n] = self.data[n]
+
+                    fits[-1].write(d)
+                    write_success=True
+                except:
+                    write_success=False
+
+                self.assertTrue(write_success,"write should not raise an error")
+                if not write_success:
+                    skipTest("cannot test result if write failed")
+
+            d = fitsio.read(fname)
+            self.compare_rec(self.data, d, "list of dicts")
+
+        finally:
+            if os.path.exists(fname):
+                #pass
+                os.remove(fname)
+
+
+    def testTableWriteDictOfArraysVar(self):
+        """
+        This version creating the table from the names and list
+        Test a basic table write, data and a header, then reading back in to
+        check the values
+        """
+
+        fname=tempfile.mktemp(prefix='fitsio-TableDictVar-',suffix='.fits')
+        try:
+            with fitsio.FITS(fname,'rw',clobber=True) as fits:
+
+                try:
+                    d={}
+                    for n in self.vardata.dtype.names:
+                        d[n] = self.vardata[n]
+
+                    fits.write(d)
+                    write_success=True
+                except:
+                    write_success=False
+
+                self.assertTrue(write_success,"write should not raise an error")
+                if not write_success:
+                    skipTest("cannot test result if write failed")
+
+            d = fitsio.read(fname)
+            self.compare_rec_with_var(self.vardata,d,"dict of arrays, var")
+
+        finally:
+            if os.path.exists(fname):
+                #pass
+                os.remove(fname)
+
+
+    def testTableWriteListOfArraysScratch(self):
+        """
+        This version creating the table from the names and list
+        Test a basic table write, data and a header, then reading back in to
+        check the values
+        """
+
+        fname=tempfile.mktemp(prefix='fitsio-TableListScratch-',suffix='.fits')
+        try:
+            with fitsio.FITS(fname,'rw',clobber=True) as fits:
+
+                try:
+                    names = [n for n in self.data.dtype.names]
+                    dlist = [self.data[n] for n in self.data.dtype.names]
+                    fits.write(dlist, names=names)
+                    write_success=True
+                except:
+                    write_success=False
+
+                self.assertTrue(write_success,"write should not raise an error")
+                if not write_success:
+                    skipTest("cannot test result if write failed")
+
+            d = fitsio.read(fname)
+            self.compare_rec(self.data, d, "list of arrays, scratch")
+
+        finally:
+            if os.path.exists(fname):
+                #pass
+                os.remove(fname)
+
+
+
+    def testTableWriteListOfArrays(self):
+        """
+        Test a basic table write, data and a header, then reading back in to
+        check the values
+        """
+
+        fname=tempfile.mktemp(prefix='fitsio-TableWriteList-',suffix='.fits')
+        try:
+            with fitsio.FITS(fname,'rw',clobber=True) as fits:
+
+                try:
+                    fits.create_table_hdu(self.data, extname='mytable')
+
+                    columns = [n for n in self.data.dtype.names]
+                    dlist = [self.data[n] for n in self.data.dtype.names]
+                    fits[-1].write(dlist, columns=columns)
+                    write_success=True
+                except:
+                    write_success=False
+
+                self.assertTrue(write_success,"write should not raise an error")
+                if not write_success:
+                    skipTest("cannot test result if write failed")
+
+            d = fitsio.read(fname, ext='mytable')
+            self.compare_rec(self.data, d, "list of arrays")
+
+        finally:
+            if os.path.exists(fname):
+                #pass
+                os.remove(fname)
+
+
+    def testTableWriteListOfArraysVar(self):
+        """
+        This version creating the table from the names and list
+        Test a basic table write, data and a header, then reading back in to
+        check the values
+        """
+
+        fname=tempfile.mktemp(prefix='fitsio-TableListScratch-',suffix='.fits')
+        try:
+            with fitsio.FITS(fname,'rw',clobber=True) as fits:
+
+                try:
+                    names = [n for n in self.vardata.dtype.names]
+                    dlist = [self.vardata[n] for n in self.vardata.dtype.names]
+                    fits.write(dlist, names=names)
+                    write_success=True
+                except:
+                    write_success=False
+
+                self.assertTrue(write_success,"write should not raise an error")
+                if not write_success:
+                    skipTest("cannot test result if write failed")
+
+            d = fitsio.read(fname)
+            self.compare_rec_with_var(self.vardata,d,"list of arrays, var")
+
+        finally:
+            if os.path.exists(fname):
+                #pass
+                os.remove(fname)
+
+
+
     def testAsciiTableWriteRead(self):
         """
         Test a basic table write, data and a header, then reading back in to
@@ -715,6 +948,34 @@ class TestReadWrite(unittest.TestCase):
                 os.remove(fname)
 
 
+    def testTableInsertColumn(self):
+        """
+        Insert a new column
+        """
+
+        fname=tempfile.mktemp(prefix='fitsio-TableInsertColumn-',suffix='.fits')
+        try:
+            with fitsio.FITS(fname,'rw',clobber=True) as fits:
+
+                fits.write_table(self.data, header=self.keys, extname='mytable')
+
+                d = fits[1].read()
+
+                for n in d.dtype.names:
+                    newname = n+'_insert'
+
+                    fits[1].insert_column(newname, d[n])
+
+                    newdata = fits[1][newname][:]
+
+                    self.compare_array(d[n], newdata, "table single field insert and read '%s'" % n)
+
+        finally:
+            if os.path.exists(fname):
+                os.remove(fname)
+
+
+
     def testSlice(self):
         """
         Test reading by slice
@@ -885,7 +1146,100 @@ class TestReadWrite(unittest.TestCase):
                 os.remove(fname)
 
 
+    def testLowerUpper(self):
+        fname=tempfile.mktemp(prefix='fitsio-LowerUpper-',suffix='.fits')
+        dt=[('MyName','f8'),('StuffThings','i4'),('Blah','f4')]
+        data=numpy.zeros(3, dtype=dt)
+        data['MyName'] = numpy.random.random(data.size)
+        data['StuffThings'] = numpy.random.random(data.size)
+        data['Blah'] = numpy.random.random(data.size)
 
+        lnames = [n.lower() for n in data.dtype.names]
+        unames = [n.upper() for n in data.dtype.names]
+
+        try:
+            with fitsio.FITS(fname,'rw',clobber=True) as fits:
+                fits.write(data)
+
+            for i in [1,2]:
+                if i == 1:
+                    lower=True
+                    upper=False
+                else:
+                    lower=False
+                    upper=True
+
+                with fitsio.FITS(fname,'rw', lower=lower, upper=upper) as fits:
+                    for rows in [None, [1,2]]:
+
+                        d=fits[1].read(rows=rows)
+                        self.compare_names(d.dtype.names,data.dtype.names,
+                                           lower=lower,upper=upper)
+
+
+                        d=fits[1].read(rows=rows, columns=['MyName','stuffthings'])
+                        self.compare_names(d.dtype.names,data.dtype.names[0:2],
+                                           lower=lower,upper=upper)
+
+                        d = fits[1][1:2]
+                        self.compare_names(d.dtype.names,data.dtype.names,
+                                           lower=lower,upper=upper)
+
+                        if rows is not None:
+                            d = fits[1][rows]
+                        else:
+                            d = fits[1][:]
+                        self.compare_names(d.dtype.names,data.dtype.names,
+                                           lower=lower,upper=upper)
+
+                        if rows is not None:
+                            d = fits[1][['myname','stuffthings']][rows]
+                        else:
+                            d = fits[1][['myname','stuffthings']][:]
+                        self.compare_names(d.dtype.names,data.dtype.names[0:2],
+                                           lower=lower,upper=upper)
+
+                # using overrides
+                with fitsio.FITS(fname,'rw') as fits:
+                    for rows in [None, [1,2]]:
+
+                        d=fits[1].read(rows=rows, lower=lower, upper=upper)
+                        self.compare_names(d.dtype.names,data.dtype.names,
+                                           lower=lower,upper=upper)
+
+
+                        d=fits[1].read(rows=rows, columns=['MyName','stuffthings'],
+                                       lower=lower,upper=upper)
+                        self.compare_names(d.dtype.names,data.dtype.names[0:2],
+                                           lower=lower,upper=upper)
+
+
+
+                for rows in [None, [1,2]]:
+                    d=fitsio.read(fname, rows=rows, lower=lower, upper=upper)
+                    self.compare_names(d.dtype.names,data.dtype.names,
+                                       lower=lower,upper=upper)
+
+                    d=fitsio.read(fname, rows=rows, columns=['MyName','stuffthings'],
+                                  lower=lower, upper=upper)
+                    self.compare_names(d.dtype.names,data.dtype.names[0:2],
+                                       lower=lower,upper=upper)
+
+
+        finally:
+            if os.path.exists(fname):
+                os.remove(fname)
+
+
+    def compare_names(self, read_names, true_names, lower=False, upper=False):
+        for nread,ntrue in zip(read_names,true_names):
+            if lower:
+                tname = ntrue.lower()
+                mess="lower: '%s' vs '%s'" % (nread,tname)
+            else:
+                tname = ntrue.upper()
+                mess="upper: '%s' vs '%s'" % (nread,tname)
+            self.assertEqual(nread, tname, mess)
 
 
     def compare_headerlist_header(self, header_list, header):
